package fr.itsonus.bousoleplussbackend.usecase;

import fr.itsonus.bousoleplussbackend.domain.auth.exception.AlreadyExistingUserException;
import fr.itsonus.bousoleplussbackend.domain.auth.model.User;
import fr.itsonus.bousoleplussbackend.domain.auth.spi.UserCacheRepository;
import fr.itsonus.bousoleplussbackend.domain.quiz.spi.QuestionCacheRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

@Service
@RequiredArgsConstructor
public class UserCreationUseCase {

    private static final String DEFAULT_BUNDLE_LABEL = "Boussole standard";
    private final UserCacheRepository userRepository;
    private final PasswordEncoder passwordEncoder;
    private final BundleUseCase bundleUseCase;
    private final QuestionCacheRepository questionCacheRepository;

    public void createUserAndDefaultBundle(final User user) {
        checkUserExistence(user);
        var newUser = userRepository.save(user.toBuilder().password(passwordEncoder.encode(user.getPassword())).build());
        bundleUseCase.create(DEFAULT_BUNDLE_LABEL, questionCacheRepository.findDefaultQuestions(), newUser);
    }

    private void checkUserExistence(User user) {
        if (userRepository.existsByEmail(user.getEmail())) {
            throw new AlreadyExistingUserException("User already registered with the e-mail provided");
        }
    }
}
